import shutil
from subprocess import call
from django.core.management.base import BaseCommand
from django.conf import settings


class Command(BaseCommand):
    help = 'Check Python files for PEP8 compliance'

    def handle(self, *args, **options):
        shutil.os.chdir(settings.SRC_PATH)
        return_value = call(["flake8", "./"])
        if return_value > 0:
            exit(return_value)



# Generated by Django 2.1.2 on 2018-11-15 09:26
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('document', '0002_auto_20180814_1946'),
    ]

    operations = [
        migrations.AlterField(
            model_name='document',
            name='doc_version',
            field=models.DecimalField(decimal_places=1, default=2.3, max_digits=3),
        ),
    ]



from django import template
from django.templatetags.static import PrefixNode
from django.apps import apps
from django.conf import settings
from allauth.socialaccount.models import providers

register = template.Library()


@register.inclusion_tag('fiduswriter/config.html', takes_context=True)
def fiduswriter_config_js(context):
    """
    Add Fidus Writer config variables to the window object in JavaScript.
    Usage::
        {% fiduswriter_config_js %}
    """
    if apps.is_installed('django.contrib.staticfiles'):
        from django.contrib.staticfiles.storage import staticfiles_storage
        static_url = staticfiles_storage.base_url
    else:
        static_url = PrefixNode.handle_simple("STATIC_URL")
    if hasattr(settings, 'WS_PORT'):
        ws_port = settings.WS_PORT
    else:
        ws_port = ''
    if hasattr(settings, 'WS_SERVER'):
        ws_server = settings.WS_SERVER
    else:
        ws_server = ''
    socialaccount_providers = []
    for provider in providers.registry.get_list():
        socialaccount_providers.append({
            'id': provider.id,
            'name': provider.name,
            'login_url': provider.get_login_url(context['request'])
        })
    return {
        'static_url': static_url,
        'ws_port': ws_port,
        'ws_server': ws_server,
        'contact_email': settings.CONTACT_EMAIL,
        'test_server': (
            'true' if settings.TEST_SERVER else 'false'
        ),
        'is_free': ('true' if settings.IS_FREE else 'false'),
        'registration_open': (
            'true' if settings.REGISTRATION_OPEN else 'false'
        ),
        'language': context['request'].LANGUAGE_CODE,
        'socialaccount_providers': socialaccount_providers,
        'debug': (
            'true' if settings.DEBUG else 'false'
        )
    }



# Generated by Django 2.1 on 2018-08-14 19:46
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('document', '0001_squashed_0032_document_listed'),
    ]

    operations = [
        migrations.AlterField(
            model_name='document',
            name='doc_version',
            field=models.DecimalField(decimal_places=1, default=2.2, max_digits=3),
        ),
    ]



from django.contrib import admin
from bibliography.models import (
    Entry,
    EntryCategory
)


class EntryCategoryAdmin(admin.ModelAdmin):
    pass


admin.site.register(EntryCategory, EntryCategoryAdmin)


class EntryAdmin(admin.ModelAdmin):
    pass


admin.site.register(Entry, EntryAdmin)



# Generated by Django 1.11.13 on 2018-08-14 17:35
from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    replaces = [('user', '0001_initial'), ('user', '0002_rename_account_tables'), ('user', '0003_auto_20151226_1110')]

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='TeamMember',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('roles', models.CharField(blank=True, max_length=100)),
                ('leader', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='leader', to=settings.AUTH_USER_MODEL)),
                ('member', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='member', to=settings.AUTH_USER_MODEL)),
            ],
        ),
        migrations.CreateModel(
            name='UserProfile',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('about', models.TextField(blank=True, max_length=500)),
                ('user', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
        ),
        migrations.AlterUniqueTogether(
            name='teammember',
            unique_together=set([('leader', 'member')]),
        ),
    ]



# Generated by Django 2.2.4 on 2019-08-28 09:08
import json
from django.db import migrations
import zipfile
import tempfile
import os


# from https://stackoverflow.com/questions/25738523/how-to-update-one-file-inside-zip-file-using-python
def update_revision_zip(zipname, citation_styles):
    # generate a temp file
    tmpfd, tmpname = tempfile.mkstemp(dir=os.path.dirname(zipname))
    os.close(tmpfd)
    document_json = False
    # create a temp copy of the archive without filename
    with zipfile.ZipFile(zipname, 'r') as zin:
        with zipfile.ZipFile(tmpname, 'w') as zout:
            zout.comment = zin.comment # preserve the comment
            for item in zin.infolist():
                if item.filename == 'document.json':
                    document_json = json.loads(zin.read(item.filename))
                else:
                    zout.writestr(item, zin.read(item.filename))
    if not document_json:
        return
    document_json['contents']['attrs']['citationstyles'] = citation_styles
    # replace with the temp archive
    os.remove(zipname)
    os.rename(tmpname, zipname)

    # now add filename with its new data
    with zipfile.ZipFile(
        zipname,
        mode='a',
        compression=zipfile.ZIP_DEFLATED
    ) as zf:
        zf.writestr(
            'document.json',
            json.dumps(document_json)
        )


def set_citationstyles(apps, schema_editor):
    DocumentTemplate = apps.get_model('document', 'DocumentTemplate')
    for doc_template in DocumentTemplate.objects.all():
        citation_styles = []
        for style in doc_template.citation_styles.all():
            style_name = style.short_title
            if style_name in [
                'american-anthropological-association',
                'apa',
                'chicago-author-date',
                'chicago-note-bibliography',
                'oxford-university-press-humsoc',
                'nature'
            ]:
                citation_styles.append(style_name)
            elif style_name == 'mla':
                citation_styles.append('modern-language-association')
            elif style_name == 'harvard1':
                citation_styles.append('harvard-cite-them-right')
            else:
                continue
        if len(citation_styles) == 8:
            # includes all styles, which is the default, so we can ignore it.
            continue
        definition = json.loads(doc_template.definition)
        definition['attrs']['citationstyles'] = citation_styles
        doc_template.definition = json.dumps(definition)
        doc_template.save()
        for doc in doc_template.document_set.all():
            contents = json.loads(doc.contents)
            contents['attrs']['citationstyles'] = citation_styles
            doc.contents = json.dumps(contents)
            doc.save()
            for revision in doc.documentrevision_set.all():
                update_revision_zip(
                    revision.file_object.path,
                    citation_styles
                )
        print('applied change')


class Migration(migrations.Migration):

    dependencies = [
        ('document', '0014_auto_20190811_1204'),
    ]

    operations = [
        migrations.RunPython(set_citationstyles),
    ]



from django.http import JsonResponse

from feedback.models import Feedback


def feedback(request):
    status = 405
    response = {}
    if request.is_ajax() and request.method == 'POST':
        status = 200
        feedback_message = request.POST['message']
        new_feedback = Feedback()
        new_feedback.message = feedback_message
        if request.user.is_authenticated:
            new_feedback.owner = request.user
        new_feedback.save()

    return JsonResponse(
        response,
        status=status
    )



# Generated by Django 2.2.3 on 2019-08-09 14:55

from django.db import migrations


class Migration(migrations.Migration):
    atomic = False
    dependencies = [
        ('document', '0013_auto_20190808_1126'),
        ('style', '0003_auto_20190803_0749'),
    ]

    operations = [
        migrations.RenameModel(
            old_name='DocumentStyle',
            new_name='OldDocumentStyle',
        ),
    ]



import os
from django.core.management.base import BaseCommand, CommandError
from django.core.management import call_command
from django.conf import settings
from django.contrib.flatpages.models import FlatPage

from document.models import DocumentTemplate
from style.models import DocumentStyle


class Command(BaseCommand):
    help = ('Setup Fidus Writer installation.')

    def add_arguments(self, parser):
        parser.add_argument(
            '--restart',
            action='store_true',
            dest='restart',
            default=False,
            help='Flush database before initialization.',
        )
        parser.add_argument(
            '--no-static',
            action='store_true',
            dest='no-static',
            default=False,
            help='Do not collect static files.',
        )
        parser.add_argument(
            '--no-compress',
            action='store_true',
            dest='no-compress',
            default=False,
            help='Do not attempt to compress static files.',
        )
        parser.add_argument(
            '--force-transpile',
            action='store_true',
            dest='force_transpile',
            default=True,
            help='Force a transpile if nothing has changed.',
        )

    def handle(self, *args, **options):
        if options["force_transpile"]:
            force_transpile = True
        else:
            force_transpile = False
        if options["restart"]:
            call_command("flush")
            call_command("migrate", fake=True)
        else:
            call_command("migrate")
        if DocumentTemplate.objects.count() == 0:
            call_command(
                "loaddata",
                os.path.join(
                    settings.SRC_PATH,
                    "document/fixtures/initial_documenttemplates.json"
                )
            )
        if DocumentStyle.objects.count() == 0:
            call_command(
                "loaddata",
                os.path.join(
                    settings.SRC_PATH,
                    "style/fixtures/initial_styles.json"
                )
            )
        if FlatPage.objects.count() == 0:
            call_command(
                "loaddata",
                os.path.join(
                    settings.SRC_PATH,
                    "base/fixtures/initial_terms.json"
                )
            )
        if (
            os.environ.get('NO_COMPILEMESSAGES') or
            (
                os.path.isfile(os.path.join(
                    settings.SRC_PATH,
                    "locale/BASE/LC_MESSAGES/django.mo"
                )) and
                os.path.getmtime(os.path.join(
                    settings.SRC_PATH,
                    "locale/BASE/LC_MESSAGES/django.mo"
                )) > os.path.getmtime(os.path.join(
                    settings.SRC_PATH,
                    "locale/BASE/LC_MESSAGES/django.po"
                ))
            )

        ):
            pass
        else:
            call_command("compilemessages")
        call_command("transpile", force=force_transpile)
        if (
            not options["no-compress"] and
            settings.COMPRESS_OFFLINE and
            settings.COMPRESS_ENABLED
        ):
            try:
                call_command("compress")
            except CommandError:
                pass
        if (
            not options["no-static"] and
            not settings.DEBUG
        ):
            call_command("collectstatic", interactive=False)



# Generated by Django 2.1.4 on 2019-01-15 21:11

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('style', '0001_squashed_0002_auto_20151226_1110'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('document', '0003_auto_20181115_0926'),
    ]

    operations = [
        migrations.CreateModel(
            name='DocumentTemplate',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(blank=True, default='', max_length=255)),
                ('definition', models.TextField(default='{}')),
                ('definition_hash', models.CharField(blank=True, default='', max_length=22)),
                ('citation_styles', models.ManyToManyField(to='style.CitationStyle')),
                ('document_styles', models.ManyToManyField(to='style.DocumentStyle')),
                ('export_templates', models.ManyToManyField(blank=True, to='document.ExportTemplate')),
                ('user', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
        ),
    ]



# Generated by Django 2.1.4 on 2019-01-15 21:11

from django.db import migrations, models
import django.db.models.deletion
import document.models

DOC_TEMPLATE = '''{
    "type": "article",
    "content": [{
        "type": "title"
    }, {
        "type": "heading_part",
        "attrs": {
            "id": "subtitle",
            "title": "Subtitle",
            "initial": [{
                "type": "heading1",
                "attrs": {
                    "id": "H6323428"
                }
            }],
            "optional": "hidden",
            "hidden": true,
            "elements": ["heading1"],
            "marks": ["strong", "em", "link", "anchor"],
            "metadata": "subtitle"
        },
        "content": [{
            "type": "heading1",
            "attrs": {
                "id": "H6323428"
            }
        }]
    }, {
        "type": "contributors_part",
        "attrs": {
            "id": "authors",
            "title": "Authors",
            "optional": "hidden",
            "hidden": true,
            "item_title": "Author",
            "metadata": "authors"
        }
    }, {
        "type": "richtext_part",
        "attrs": {
            "id": "abstract",
            "title": "Abstract",
            "optional": "hidden",
            "hidden": true,
            "elements": ["paragraph", "heading1", "heading2", "heading3", "heading4", "heading5", "heading6", "figure", "ordered_list", "bullet_list", "horizontal_rule", "equation", "citation", "blockquote", "footnote", "table"],
            "marks": ["strong", "em", "link", "anchor"],
            "metadata": "abstract"
        },
        "content": [{
            "type": "paragraph"
        }]
    }, {
        "type": "tags_part",
        "attrs": {
            "id": "keywords",
            "title": "Keywords",
            "optional": "hidden",
            "hidden": true,
            "item_title": "Keyword",
            "metadata": "keywords"
        }
    }, {
        "type": "richtext_part",
        "attrs": {
            "id": "body",
            "title": "Body",
            "elements": ["paragraph", "heading1", "heading2", "heading3", "heading4", "heading5", "heading6", "figure", "ordered_list", "bullet_list", "horizontal_rule", "equation", "citation", "blockquote", "footnote", "table"],
            "marks": ["strong", "em", "link", "anchor"]
        },
        "content": [{
            "type": "paragraph"
        }]
    }],
    "attrs": {
        "footnote_elements": ["paragraph", "heading1", "heading2", "heading3", "heading4", "heading5", "heading6", "figure", "ordered_list", "bullet_list", "horizontal_rule", "equation", "citation", "blockquote", "table"],
        "footnote_marks": ["strong", "em", "link", "anchor"],
        "languages": ["af-ZA", "sq-AL", "ar", "ast", "be", "br", "bg", "ca", "ca-ES-Valencia", "zh-CN", "da", "nl", "en-AU", "en-CA", "en-NZ", "en-ZA", "en-GB", "en-US", "eo", "fr", "gl", "de-DE", "de-AU", "de-CH", "el", "he", "is", "it", "ja", "km", "lt", "ml", "nb-NO", "nn-NO", "fa", "pl", "pt-BR", "pt-PT", "ro", "ru", "tr", "sr-SP-Cy", "sr-SP-Lt", "sk", "sl", "es", "sv", "ta", "tl", "uk"],
        "papersizes": ["A4", "US Letter"],
        "template": "Standard Article",
        "papersize": "A4",
        "language": "af-ZA",
        "bibliography_header": {}
    }
}
'''


DOC_TEMPLATE_HASH = '9oh5bHynenFdZxCA5NvXVQ'


DEFAULT_TEMPLATE_ID = 1


def create_default_template(apps, schema_editor):
    global DEFAULT_TEMPLATE_ID
    Document = apps.get_model('document', 'Document')
    document = Document.objects.first()
    if not document:
        # There are no documents in the system yet. CitationStyles/
        # DocumentStyles will not have been created yet either. We therefore do
        # not need to create any DocumentTemplates just yet. A template will
        # be created automatically when creating the first Document.
        return
    DocumentTemplate = apps.get_model('document', 'DocumentTemplate')
    template = DocumentTemplate.objects.first()
    if template:
        DEFAULT_TEMPLATE_ID = template.pk
        return
    template = DocumentTemplate()
    template.definition = DOC_TEMPLATE
    template.definition_hash = DOC_TEMPLATE_HASH
    template.title = 'Standard Article'
    template.save()
    CitationStyle = apps.get_model('style', 'CitationStyle')
    for style in CitationStyle.objects.all():
        template.citation_styles.add(style)
    DocumentStyle = apps.get_model('style', 'DocumentStyle')
    for style in DocumentStyle.objects.all():
        template.document_styles.add(style)
    ExportTemplate = apps.get_model('document', 'ExportTemplate')
    for exporter in ExportTemplate.objects.all():
        template.export_templates.add(exporter)
    DEFAULT_TEMPLATE_ID = template.pk
    return


class Migration(migrations.Migration):

    dependencies = [
        ('document', '0004_documenttemplate'),
    ]

    operations = [
        migrations.RunPython(create_default_template),
        migrations.AddField(
            model_name='document',
            name='template',
            field=models.ForeignKey(default=DEFAULT_TEMPLATE_ID, on_delete=django.db.models.deletion.CASCADE, to='document.DocumentTemplate'),
        ),
    ]



# Generated by Django 2.2.3 on 2019-08-09 15:57
import os
import shutil
from django.db import migrations
from django.conf import settings
from django.core.files import File

def convert_old_models(apps, schema_editor):
    DocumentTemplate = apps.get_model('document', 'DocumentTemplate')
    DocumentStyle = apps.get_model('style', 'DocumentStyle')
    DocumentStyleFile = apps.get_model('style', 'DocumentStyleFile')
    ExportTemplate = apps.get_model('style', 'ExportTemplate')
    if not os.path.exists(
        os.path.join(
            settings.MEDIA_ROOT,
            "style-files"
        )
    ):
        os.makedirs(
            os.path.join(
                settings.MEDIA_ROOT,
                "style-files"
            )
        )
    if not os.path.exists(
        os.path.join(
            settings.MEDIA_ROOT,
            "export-template-files"
        )
    ):
        os.makedirs(
            os.path.join(
                settings.MEDIA_ROOT,
                "export-template-files"
            )
        )
    for dt in DocumentTemplate.objects.all():
        for old_ds in dt.document_styles.all():
            doc_style = DocumentStyle()
            doc_style.title = old_ds.title
            slug = old_ds.filename
            existing_ds = DocumentStyle.objects.filter(
                document_template=dt,
                slug=slug
            ).first()
            ds_count = 0
            while existing_ds:
                slug = str(ds_count)
                ds_count += 1
                existing_ds = DocumentStyle.objects.filter(
                    document_template=dt,
                    slug=slug
                ).first()
            doc_style.slug = slug
            doc_style.contents = old_ds.contents
            doc_style.document_template = dt
            doc_style.save()
            for font in old_ds.fonts.all():
                style_file = DocumentStyleFile()
                filename = font.font_file.path.split('/').pop()
                existing_dsf = DocumentStyleFile.objects.filter(
                    style=doc_style,
                    filename=filename
                ).first()
                dsf_count = 0
                while existing_dsf:
                    filename = str(dsf_count)
                    dsf_count += 1
                    existing_dsf = DocumentStyleFile.objects.filter(
                        style=doc_style,
                        filename=filename
                    ).first()
                new_file = File(open(font.font_file.path, 'rb'))
                style_file.filename = filename
                style_file.style = doc_style
                style_file.file.save(
                    filename,
                    new_file
                )
                style_file.save()
                doc_style.contents = (
                    '@font-face {' +
                    font.fontface_definition.replace(
                        '[URL]',
                        style_file.filename
                    ) +
                    '}\n\n' + doc_style.contents
                )
                doc_style.save()
        for old_et in dt.export_templates.all():
            export_template = ExportTemplate()
            title = old_et.file_name[:128]
            existing_et = ExportTemplate.objects.filter(
                document_template=dt,
                title=title
            ).first()
            et_count = 0
            while existing_et:
                title = str(et_count)
                et_count += 1
                existing_et = ExportTemplate.objects.filter(
                    document_template=dt,
                    title=title
                ).first()
            export_template.title = title
            export_template.file_type = old_et.file_type
            export_template.document_template = dt
            new_file = File(open(old_et.template_file.path, 'rb'))
            export_template.template_file.save(
                title + '.' + old_et.file_type,
                new_file
            )
            # export_template.template_file = old_et.template_file
            export_template.save()


def empty_new_models(apps, schema_editor):
    DocumentStyle = apps.get_model('style', 'DocumentStyle')
    ExportTemplate = apps.get_model('style', 'ExportTemplate')
    DocumentStyle.objects.all().delete()
    ExportTemplate.objects.all().delete()
    if os.path.exists(
        os.path.join(
            settings.MEDIA_ROOT,
            "style-files"
        )
    ):
        shutil.rmtree(
            os.path.join(
                settings.MEDIA_ROOT,
                "style-files"
            )
        )
    if os.path.exists(
        os.path.join(
            settings.MEDIA_ROOT,
            "export-template-files"
        )
    ):
        shutil.rmtree(
            os.path.join(
                settings.MEDIA_ROOT,
                "export-template-files"
            )
        )


class Migration(migrations.Migration):

    dependencies = [
        ('style', '0005_documentstyle_documentstylefile_exporttemplate'),
    ]

    operations = [
        migrations.RunPython(convert_old_models, empty_new_models),
    ]



# Generated by Django 1.11.13 on 2018-08-14 17:37
from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import usermedia.models


class Migration(migrations.Migration):

    replaces = [('usermedia', '0001_initial'), ('usermedia', '0002_auto_20151226_1110'), ('usermedia', '0003_auto_20160218_2222'), ('usermedia', '0004_auto_20160218_2250'), ('usermedia', '0005_userimage'), ('usermedia', '0006_auto_20170813_2154'), ('usermedia', '0007_auto_20170813_2206'), ('usermedia', '0008_documentimage'), ('usermedia', '0009_auto_20170908_0953')]

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('document', '0030_document_bibliography'),
    ]

    operations = [
        migrations.CreateModel(
            name='Image',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(max_length=128)),
                ('added', models.DateTimeField(auto_now_add=True)),
                ('image', models.FileField(upload_to=usermedia.models.get_file_path)),
                ('thumbnail', models.ImageField(blank=True, max_length=500, null=True, upload_to='image_thumbnails')),
                ('image_cat', models.CharField(default='', max_length=255)),
                ('file_type', models.CharField(blank=True, max_length=20, null=True)),
                ('height', models.IntegerField(blank=True, null=True)),
                ('width', models.IntegerField(blank=True, null=True)),
                ('checksum', models.BigIntegerField(default=0)),
                ('uploader', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='uploader', to=settings.AUTH_USER_MODEL)),
            ],
        ),
        migrations.CreateModel(
            name='ImageCategory',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('category_title', models.CharField(max_length=100)),
                ('category_owner', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'verbose_name_plural': 'Image categories',
            },
        ),
        migrations.AddField(
            model_name='image',
            name='owner',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='image_owner', to=settings.AUTH_USER_MODEL),
        ),
        migrations.AlterField(
            model_name='image',
            name='uploader',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='image_uploader', to=settings.AUTH_USER_MODEL),
        ),
        migrations.CreateModel(
            name='UserImage',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(max_length=128)),
                ('image_cat', models.CharField(default='', max_length=255)),
                ('image', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='usermedia.Image')),
                ('owner', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='image_owner', to=settings.AUTH_USER_MODEL)),
            ],
        ),
        migrations.RemoveField(
            model_name='image',
            name='image_cat',
        ),
        migrations.RemoveField(
            model_name='image',
            name='owner',
        ),
        migrations.RemoveField(
            model_name='image',
            name='title',
        ),
        migrations.CreateModel(
            name='DocumentImage',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(default='', max_length=128)),
                ('document', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='document.Document')),
                ('image', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='usermedia.Image')),
            ],
        ),
    ]



from django.conf.urls import url

from . import views

urlpatterns = [
    url(
        '^documentlist/$',
        views.get_documentlist,
        name='get_documentlist'
    ),
    url(
        '^documentlist/extra/$',
        views.get_documentlist_extra,
        name='get_documentlist_extra'
    ),
    url('^delete/$', views.delete, name='delete'),
    url(
        '^create_doc/(?P<template_id>[0-9]+)/$',
        views.create_doc,
        name='create_doc'
    ),
    url('^import/create/$', views.import_create, name='import_create'),
    url('^import/image/$', views.import_image, name='import_image'),
    url('^import/$', views.import_doc, name='import_doc'),
    url('^upload/$', views.upload_revision, name='upload_revision'),
    url(
        '^get_revision/(?P<revision_id>[0-9]+)/$',
        views.get_revision,
        name='get_revision'
    ),
    url(
        '^delete_revision/$',
        views.delete_revision,
        name='delete_revision'
    ),
    url(
        '^get_access_rights/$',
        views.get_access_rights,
        name='get_access_rights'
    ),
    url(
        '^save_access_rights/$',
        views.save_access_rights,
        name='save_access_rights'
    ),
    url(
        '^invite/$',
        views.invite,
        name='invite'
    ),
    url(
        '^comment_notify/$',
        views.comment_notify,
        name='comment_notify'
    ),
    url(
        '^admin/get_template_extras/$',
        views.get_template_extras,
        name='get_template_extras'
    ),
    url(
        '^admin/get_all_old/$',
        views.get_all_old_docs,
        name='get_all_old_docs'
    ),
    url(
        '^admin/save_doc/$',
        views.save_doc,
        name='save_doc'
    ),
    url(
        '^admin/add_images_to_doc/$',
        views.add_images_to_doc,
        name='add_images_to_doc'
    ),
    url(
        '^admin/get_all_revision_ids/$',
        views.get_all_revision_ids,
        name='get_all_revision_ids'
    ),
    url(
        '^admin/get_all_template_ids/$',
        views.get_all_template_ids,
        name='get_all_template_ids'
    ),
    url(
        '^admin/get_template/$',
        views.get_template,
        name='get_template'
    ),
    url(
        '^admin/save_template/$',
        views.save_template,
        name='save_template'
    ),
    url(
        '^admin/get_user_biblist/$',
        views.get_user_biblist,
        name='get_user_biblist'
    ),
    url(
        '^admin/update_revision/$',
        views.update_revision,
        name='update_revision'
    )
]



import os
from testing.testcases import LiveTornadoTestCase
from testing.selenium_helper import SeleniumHelper
from selenium.webdriver.common.keys import Keys
from selenium.webdriver.common.by import By
from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.support import expected_conditions as EC

from django.conf import settings


class BibliographyOverviewTest(LiveTornadoTestCase, SeleniumHelper):

    @classmethod
    def setUpClass(cls):
        super(BibliographyOverviewTest, cls).setUpClass()
        cls.base_url = cls.live_server_url
        driver_data = cls.get_drivers(1)
        cls.driver = driver_data["drivers"][0]
        cls.client = driver_data["clients"][0]
        cls.driver.implicitly_wait(driver_data["wait_time"])
        cls.wait_time = driver_data["wait_time"]

    @classmethod
    def tearDownClass(cls):
        cls.driver.quit()
        super(BibliographyOverviewTest, cls).tearDownClass()

    def setUp(self):
        self.verificationErrors = []
        self.accept_next_alert = True
        self.user = self.create_user(
            username='Yeti',
            email='yeti@snowman.com',
            passtext='otter1'
        )
        self.login_user(self.user, self.driver, self.client)

    def test_overview(self):
        driver = self.driver
        driver.get(self.base_url + "/bibliography/")
        driver.find_element_by_xpath(
            "(.//*[normalize-space(text()) and normalize-space(.)="
            "'All categories'])[1]/following::button[1]"
        ).click()
        driver.find_element_by_xpath(
            "(.//*[normalize-space(text()) and normalize-space(.)="
            "'Edit Categories'])[1]/following::input[1]"
        ).click()
        driver.find_element_by_xpath(
            "(.//*[normalize-space(text()) and normalize-space(.)="
            "'Edit Categories'])[1]/following::input[1]"
        ).clear()
        driver.find_element_by_xpath(
            "(.//*[normalize-space(text()) and normalize-space(.)="
            "'Edit Categories'])[1]/following::input[1]"
        ).send_keys("Fish")
        driver.find_element_by_xpath(
            "(.//*[normalize-space(text()) and normalize-space(.)="
            "'Edit Categories'])[1]/following::span[3]"
        ).click()
        driver.find_element_by_xpath(
            "(.//*[normalize-space(text()) and normalize-space(.)="
            "'Edit Categories'])[1]/following::input[2]"
        ).click()
        driver.find_element_by_xpath(
            "(.//*[normalize-space(text()) and normalize-space(.)="
            "'Edit Categories'])[1]/following::input[2]"
        ).clear()
        driver.find_element_by_xpath(
            "(.//*[normalize-space(text()) and normalize-space(.)="
            "'Edit Categories'])[1]/following::input[2]"
        ).send_keys("Table")
        driver.find_element_by_xpath(
            "(.//*[normalize-space(text()) and normalize-space(.)="
            "'Edit Categories'])[1]/following::span[4]"
        ).click()
        driver.find_element_by_xpath(
            "(.//*[normalize-space(text()) and normalize-space(.)="
            "'Edit Categories'])[1]/following::input[3]"
        ).click()
        driver.find_element_by_xpath(
            "(.//*[normalize-space(text()) and normalize-space(.)="
            "'Edit Categories'])[1]/following::input[3]"
        ).clear()
        driver.find_element_by_xpath(
            "(.//*[normalize-space(text()) and normalize-space(.)="
            "'Edit Categories'])[1]/following::input[3]"
        ).send_keys("Jungle")
        driver.find_element_by_xpath(
            "(.//*[normalize-space(text()) and normalize-space(.)="
            "'Edit Categories'])[1]/following::button[2]"
        ).click()
        driver.find_element_by_xpath(
            "(.//*[normalize-space(text()) and normalize-space(.)="
            "'All categories'])[1]/following::button[1]"
        ).click()
        try:
            self.assertEqual("Fish", driver.find_element_by_id(
                "categoryTitle_1").get_attribute("value"))
        except AssertionError as e:
            self.verificationErrors.append(str(e))
        try:
            self.assertEqual("Table", driver.find_element_by_id(
                "categoryTitle_2").get_attribute("value"))
        except AssertionError as e:
            self.verificationErrors.append(str(e))
        driver.find_element_by_xpath(
            "(.//*[normalize-space(text()) and normalize-space(.)="
            "'Edit Categories'])[1]/following::button[2]"
        ).click()
        driver.find_element_by_xpath(
            "(.//*[normalize-space(text()) and normalize-space(.)="
            "'Edit categories'])[1]/following::button[1]"
        ).click()
        driver.find_element_by_id("select-bibtype").click()
        driver.find_element_by_css_selector(
            "#select-bibtype option[value=article]").click()
        title_of_publication = driver.find_element_by_css_selector(
            ".journaltitle .ProseMirror")
        title_of_publication.click()
        title_of_publication.send_keys("Title of publication")
        title = driver.find_element_by_css_selector(".title .ProseMirror")
        title.click()
        title.send_keys("The title")
        first_name = driver.find_element_by_css_selector(".given .ProseMirror")
        first_name.click()
        first_name.click()  # TODO: We should not need two clicks
        first_name.send_keys("Hans")
        last_name = driver.find_element_by_css_selector(".family .ProseMirror")
        last_name.click()
        last_name.send_keys("Hansen")
        date_field = driver.find_element_by_css_selector("input.date")
        date_field.click()
        date_field.send_keys("1984")
        driver.find_element_by_link_text("Categories").click()
        driver.find_element_by_xpath(
            "(.//*[normalize-space(text()) and normalize-space(.)="
            "'Categories'])[2]/following::div[1]"
        ).click()
        driver.find_element_by_xpath(
            "(.//*[normalize-space(text()) and normalize-space(.)="
            "'Table'])[1]/following::div[1]"
        ).click()
        try:
            self.assertEqual("Jungle", driver.find_element_by_xpath(
                "(.//*[normalize-space(text()) and normalize-space(.)="
                "'Table'])[1]/following::div[1]"
            ).text)
        except AssertionError as e:
            self.verificationErrors.append(str(e))
        driver.find_element_by_css_selector(
            "button.fw-dark"
        ).click()
        # Closed citation dialog
        search_input = driver.find_element_by_css_selector(
            ".fw-overview-menu-item .fw-button input[type=text]")
        search_input.click()
        search_input.send_keys("women")
        try:
            self.assertEqual(
                "No sources registered",
                driver.find_element_by_css_selector(
                    ".fw-data-table.fw-large.dataTable-table td"
                ).text
            )
        except AssertionError as e:
            self.verificationErrors.append(str(e))
        search_input.send_keys(Keys.BACKSPACE)
        search_input.send_keys(Keys.BACKSPACE)
        search_input.send_keys(Keys.BACKSPACE)
        search_input.send_keys(Keys.BACKSPACE)
        search_input.send_keys(Keys.BACKSPACE)
        try:
            self.assertEqual(
                "The title",
                driver.find_element_by_css_selector(
                    ".fw-data-table-title .edit-bib").text
            )
        except AssertionError as e:
            self.verificationErrors.append(str(e))
        driver.find_element_by_css_selector(
            "button[title='Upload BibTeX file']").click()
        # bibliography path
        bib_path = os.path.join(
            settings.PROJECT_PATH,
            'bibliography/tests/uploads/bibliography.bib'
        )
        driver.find_element_by_id("bib-uploader").send_keys(bib_path)
        driver.find_element_by_css_selector(
            "button.submit-import").click()
        book_title_el = WebDriverWait(driver, self.wait_time).until(
            EC.presence_of_element_located(
                (
                    By.CSS_SELECTOR,
                    ".fw-data-table tr:nth-child(2) .edit-bib"
                )
            )
        )
        self.assertEqual(
            'Lean UX: Applying lean principles to improve user experience',
            book_title_el.text
        )

    def tearDown(self):
        self.driver.quit()
        self.assertEqual([], self.verificationErrors)



from django.db import models
from django.utils.translation import ugettext as _


class DocumentStyle(models.Model):
    title = models.CharField(
        max_length=128,
        help_text='The human readable title.',
        default='Default'
    )
    slug = models.SlugField(
        max_length=20,
        help_text='The base of the filenames the style occupies.',
        default='default'
    )
    contents = models.TextField(
        help_text='The CSS style definiton.',
        default=''
    )
    document_template = models.ForeignKey(
        'document.DocumentTemplate',
        on_delete=models.deletion.CASCADE
    )

    def __str__(self):
        return self.title

    class Meta(object):
        unique_together = (("slug", "document_template"),)


def documentstylefile_location(instance, filename):
    # preserve the original filename
    instance.filename = filename
    return '/'.join(['style-files', filename])


class DocumentStyleFile(models.Model):
    file = models.FileField(
        upload_to=documentstylefile_location,
        help_text=(
            'A file references in the style. The filename will be replaced '
            'with the final url of the file in the style.'
        )
    )
    filename = models.CharField(
        max_length=255,
        help_text='The original filename.'
    )
    style = models.ForeignKey(
        'DocumentStyle',
        on_delete=models.deletion.CASCADE
    )

    def __str__(self):
        return self.filename + ' of ' + self.style.title

    def natural_key(self):
        return (self.file.url, self.filename)

    class Meta(object):
        unique_together = (("filename", "style"),)


TEMPLATE_CHOICES = (
    ('docx', 'DOCX'),
    ('odt', 'ODT')
)


def template_filename(instance, filename):
    instance.title = filename.split('.')[0]
    return '/'.join(['export-template-files', filename])


class ExportTemplate(models.Model):
    template_file = models.FileField(upload_to=template_filename)
    title = models.CharField(
        max_length=128,
        help_text='The human readable title.',
        default=_('Default')
    )
    file_type = models.CharField(
        max_length=5,
        choices=TEMPLATE_CHOICES,
        blank=False)
    document_template = models.ForeignKey(
        'document.DocumentTemplate',
        on_delete=models.deletion.CASCADE
    )

    def __str__(self):
        return self.title + " (" + self.file_type + ")"

    class Meta(object):
        unique_together = (("title", "document_template"),)



"""
Contains base templates (html),
statics that are shared among different django applications
Also contains helper for starting tornado server and management commands
"""
default_app_config = 'base.apps.BaseConfig'



# Generated by Django 2.1.7 on 2019-07-01 09:43
import os
from shutil import copyfile
from zipfile import ZipFile

from django.db import migrations
from django.core.files import File
from django.conf import settings


def find_file_path(file_path):
    zip = ZipFile(file_path, 'r')
    if len(zip.read('bibliography.json')) > 0:
        # bibliography is not broken, use this file
        return file_path
    path_parts = file_path.split('/')
    if len(path_parts) < 5 or path_parts[-5] != 'revision':
        # We could not find a working version
        return None
    file_datetime = zip.getinfo('mimetype').date_time
    path_parts.pop()
    path_parts.pop()
    path_parts.pop()
    dir_path = '/'.join(path_parts)
    for file in os.listdir(dir_path):
        file_path = os.path.join(dir_path, file)
        if not os.path.isfile(file_path):
            continue
        zip = ZipFile(file_path, 'r')
        if zip.getinfo('mimetype').date_time == file_datetime:
            return find_file_path(file_path)
    return None


def fix_document_revisions(apps, schema_editor):
    # Upgraded document revisions have been saved without bibliography and in
    # odd location in FW 3.5 + 3.6. Luckily previous versions of the revisions
    # have not been overwritten by newer versions so we switch to the older
    # versions in the revision folder and put them into the document-revisions
    # folder (new location).
    DocumentRevision = apps.get_model('document', 'DocumentRevision')
    revisions = DocumentRevision.objects.filter(
        file_object__contains='revision/'
    )
    for revision in revisions:
        old_path = find_file_path(revision.file_object.path)
        if not old_path:
            print(
                "Document revision {id} could not be fixed".format(
                    id=revision.pk
                )
            )
            continue
        # We create a backup just in case
        if not os.path.exists(
            os.path.join(
                settings.MEDIA_ROOT,
                "document-revisions-backup"
            )
        ):
            os.makedirs(
                os.path.join(
                    settings.MEDIA_ROOT,
                    "document-revisions-backup"
                )
            )
        copyfile(
            old_path,
            os.path.join(
                settings.MEDIA_ROOT,
                "document-revisions-backup/{id}.fidus".format(id=revision.pk)
            )
        )
        new_file = File(open(old_path, 'rb'))
        revision.file_object.save(
            "document-revisions/{id}.fidus".format(id=revision.pk),
            new_file
        )


class Migration(migrations.Migration):

    dependencies = [
        ('document', '0007_auto_20190227_2105'),
    ]

    operations = [
        migrations.RunPython(fix_document_revisions),
    ]



from django.shortcuts import render
from django.views.decorators.csrf import ensure_csrf_cookie
from django.contrib.sites.shortcuts import get_current_site
from django.contrib.flatpages.models import FlatPage
from django.http import JsonResponse


@ensure_csrf_cookie
def app(request):
    """
    Load a page controlled by the JavaScript app.
    Used all user facing pages after login.
    """
    return render(request, 'app.html')


def manifest_json(request):
    """
    Load the manifest.json.
    """
    return render(request, 'manifest.json')


# view is shown only in admin interface, so authentication is taken care of
def admin_console(request):
    """
    Load the admin console page.
    """
    return render(request, 'admin/console.html')


def flatpage(request):
    """
    Models: `flatpages.flatpages`
    Context:
        flatpage
            `flatpages.flatpages` object
    """
    response = {}
    status = 404
    if request.is_ajax() and request.method == 'POST':
        url = request.POST['url']
        site_id = get_current_site(request).id
        flatpage = FlatPage.objects.filter(url=url, sites=site_id).first()
        if flatpage:
            status = 200
            response['title'] = flatpage.title
            response['content'] = flatpage.content
    return JsonResponse(
        response,
        status=status
    )



from django.core import serializers
from django.db import models

PythonSerializer = serializers.get_serializer("python")


class PythonWithURLSerializer(PythonSerializer):
    def handle_field(self, obj, field):
        value = field.value_from_object(obj)
        if isinstance(field, models.FileField) and hasattr(value, 'url'):
            self._current[field.name] = value.url
        else:
            return super(PythonWithURLSerializer, self).handle_field(
                obj,
                field
            )

    def end_object(self, obj):
        # We add reverse foreign key relations if they are explicitly added.
        # Needed to include DocumentStyleFont with DocumentStyle\
        if self.selected_fields is not None:
            for field in obj._meta.related_objects:
                if not field.one_to_many:
                    continue
                if field.related_name:
                    related_name = field.related_name
                else:
                    related_name = field.name + '_set'
                if related_name not in self.selected_fields:
                    continue
                if self.use_natural_foreign_keys and hasattr(
                    field.remote_field.model, 'natural_key'
                ):
                    def o2m_value(value):
                        return value.natural_key()
                else:
                    def o2m_value(value):
                        return self._value_from_field(value, value._meta.pk)
                self._current[related_name] = [
                    o2m_value(related) for related in getattr(
                        obj, related_name
                    ).iterator()
                ]

        return super(PythonWithURLSerializer, self).end_object(obj)
        # if field.remote_field.through._meta.auto_created:



# Generated by Django 2.2.4 on 2019-08-30 04:27

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('document', '0017_remove_documenttemplate_citation_styles'),
        ('style', '0007_auto_20190811_1204'),
    ]

    operations = [
        migrations.DeleteModel(
            name='CitationLocale',
        ),
        migrations.DeleteModel(
            name='CitationStyle',
        ),
    ]



import time
from random import randrange
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.common.by import By
from selenium.webdriver.support.ui import WebDriverWait
from testing.selenium_helper import SeleniumHelper
from document.models import Document


class EditorHelper(SeleniumHelper):
    """
    Common functions used in threaded tests
    """

    def create_new_document(self):
        doc = Document.objects.create(
            owner=self.user,
            template_id=1  # from fixture
        )
        doc.save()
        return doc

    def load_document_editor(self, driver, doc):
        driver.get("%s%s" % (
            self.live_server_url,
            doc.get_absolute_url()
        ))
        WebDriverWait(driver, self.wait_time).until(
            EC.presence_of_element_located((By.CLASS_NAME, 'editor-toolbar'))
        )
        self.inject_helpers(driver)

    def inject_helpers(self, driver):
        test_caret_script = open(
            'static-transpile/js/test_caret.js',
            'r'
        ).read()
        driver.execute_script(
            test_caret_script
        )

    def input_text(self, document_input, text):
        for char in text:
            document_input.send_keys(char)
            time.sleep(randrange(10, 40) / 200.0)

    def add_title(self, driver):
        title = "My title"
        driver.execute_script(
            'window.testCaret.setSelection(2,2)')
        document_input = self.driver.find_element_by_class_name(
            'ProseMirror'
        )
        self.input_text(document_input, title)

    def wait_for_doc_size(self, driver, size, seconds=False):
        if seconds is False:
            seconds = self.wait_time
        doc_size = driver.execute_script(
            'return window.theApp.page.view.state.doc.content.size')
        if doc_size < size and seconds > 0:
            time.sleep(0.1)
            self.wait_for_doc_size(driver, size, seconds - 0.1)

    def wait_for_doc_sync(self, driver, driver2, seconds=False):
        if seconds is False:
            seconds = self.wait_time
        doc_str = driver.execute_script(
            'return window.theApp.page.view.state.doc.toString()')
        doc2_str = driver2.execute_script(
            'return window.theApp.page.view.state.doc.toString()')
        if (doc_str != doc2_str):
            # The strings don't match.
            time.sleep(0.1)
            self.wait_for_doc_sync(driver, driver2, seconds - 0.1)



import base64

from builtins import object
from django.contrib import auth
from django.conf import settings
from django.core.handlers.wsgi import WSGIRequest
from importlib import import_module
from tornado.wsgi import WSGIContainer


class DjangoHandlerMixin(object):
    def get_django_session(self):
        if not hasattr(self, '_session'):
            engine = \
                import_module(settings.SESSION_ENGINE)
            session_key = \
                self.get_cookie(settings.SESSION_COOKIE_NAME)
            self._session = engine.SessionStore(session_key)
        return self._session

    def get_current_user(self):
        # get_user needs a django request object, but only looks at the session

        class Dummy(object):
            pass

        django_request = Dummy()
        django_request.session = self.get_django_session()
        user = auth.get_user(django_request)
        if user.is_authenticated:
            return user
        else:
            # try basic auth
            if 'Authorization' not in self.request.headers:
                return None
            (kind, data) = self.request.headers['Authorization'].split(' ')
            if kind != 'Basic':
                return None
            data += "=" * ((4 - len(data) % 4) % 4)
            (username, password) = base64.b64decode(
                data
            ).decode('utf-8').split(':')
            user = auth.authenticate(username=username, password=password)
            if user is not None and user.is_authenticated:
                return user
            return None

    def get_django_request(self):
        request = \
            WSGIRequest(WSGIContainer.environ(self.request))
        request.session = self.get_django_session()

        if self.current_user:
            request.user = self.current_user
        else:
            request.user = auth.models.AnonymousUser()
        return request



import shutil
import os
from subprocess import call

from django.core.management.base import BaseCommand
from django.core.management import call_command
from django.conf import settings


class Command(BaseCommand):
    help = 'Check JavaScript files with ESLint'

    def handle(self, *args, **options):
        call_command("npm_install")
        shutil.os.chdir(settings.SRC_PATH)
        return_value = call([
            os.path.join(
                settings.PROJECT_PATH,
                ".transpile/node_modules/.bin/eslint"
            ),
            "."
        ])
        if return_value > 0:
            exit(return_value)



from django.core.management import call_command
from npm_mjs.management.commands.collectstatic import (
    Command as CollectStaticCommand
)


class Command(CollectStaticCommand):
    def handle(self, *args, **options):
        call_command("transpile")
        return super(Command, self).handle(*args, **options)
